const state = {
    cartItems: [],
    cartTotal: 0
}

const getters = {
    /**
     * 
     * @param {*} state 
     */
    cartItems(state) {
        return state.cartItems
    },
    /**
     * 
     * @param {*} state 
     */
    cartTotal(state) {
        return state.cartTotal
    }
}

const mutations = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setCartItems(state, payload) {
        state.cartItems= [...payload]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    pushToCart(state, payload) {
        let productFound = state.cartItems.find(item => item.id === payload.id)

        if(productFound) {
            productFound.quantity = productFound.quantity + 1
            return
        }

        state.cartItems = [payload, ...state.cartItems]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    pullFromCart(state, payload) {
        let foundIndex = state.cartItems.findIndex(item => item.id === payload.item.id)

        if (payload.mode === 'partial') {
            state.cartItems[foundIndex].quantity = state.cartItems[foundIndex].quantity - 1
        } else {
            state.cartItems.splice(foundIndex, 1)
        }
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setCartTotal(state, payload) {
        state.cartTotal = payload
    }
}

const actions = {
    /**
     * Add item to cart or increment item quantity if exist
     * @param {*} param0 
     * @param {*} payload 
     */
    addToCart({ commit }, payload) {
        commit('pushToCart', payload)
    },
    /**
     * Remove item from cart
     * @param {*} param0 
     * @param {*} payload 
     */
    removeFromCart({ commit }, payload) {
        commit('pullFromCart', {
            item: payload,
            mode: 'full'
        })
    },
    /**
     * Decrement item quantity from cart
     * @param {*} param0 
     * @param {*} payload 
     */
    decreaseFromCart({ commit }, payload) {
        commit('pullFromCart', {
            item: payload,
            mode: 'partial'
        })
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    updateCartTotal({ commit }, payload) {
        commit('setCartTotal', payload)
    },
    /**
     * 
     * @param {*} param0 
     */
    resetCart({ commit }) {
        commit('setCartItems', [])
        commit('setCartTotal', 0)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}